<?php
/**
 * Copyright 2014-2018 Cherry Checkout SA
 *
 * NOTICE OF LICENSE
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  @author    Cherry Checkout SA <info@cherrycheckout.com>
 *  @copyright 2014-2018 Cherry Checkout SA
 *  @license   https://opensource.org/licenses/BSD-3-Clause  3-Clause BSD License (BSD-3-Clause)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once('CherryCheckoutAuth.php');

class CherryCheckoutApi
{
    private $api;
    private $publicKey;
    private $privateKey;

    private $me;

    public function __construct($api, $publicKey, $privateKey)
    {
        $this->api = $api;
        $this->publicKey = $publicKey;
        $this->privateKey = $privateKey;
    }

    public function me($forceFromApi = false)
    {
        if ($this->me && !$forceFromApi) {
            return $this->me;
        }

        try {
            $me = $this->get('/app/me');
            return $me;
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    public function updateMe($values)
    {
        try {
            $this->post('PUT', '/app/me', $values);
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    public function createOrder($order)
    {
        try {
            $this->post('POST', '/orders', $order);
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    public function updateOrder($trackId, $order)
    {
        try {
            $this->post('PUT', '/orders/'.$trackId, $order);
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    public function getLastPluginVersion($cms)
    {
        try {
            $res = $this->get('/pluginVersion/'.$cms);
            return $res->lastVersion;
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    private function getRequestHeaders()
    {
        $timestamp = time();

        // Crypt the (publicKey + timestamp) with the privateKey, in a sha256 algo
        $myKeyEncrypted = CherryCheckoutAuth::hashKeys($this->publicKey, $this->privateKey, $timestamp);

        return array(
            'Content-type: application/json',
            'authorization: '.$this->publicKey.':'.$myKeyEncrypted,
            'timestamp: '.$timestamp
        );
    }

    private function get($route)
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->api . $route);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->getRequestHeaders());
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        // If httpCode is not 200, return an error
        if ($httpCode != 200) {
            if ($response) {
                throw new \Exception($response);
            } else {
                throw new \Exception('No response from API : ' . curl_error($ch));
            }
        }

        return json_decode($response);
    }

    private function post($method, $route, $params)
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->api . $route);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->getRequestHeaders());
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        // If httpCode is not 200, return an error
        if ($httpCode != 200) {
            if ($response) {
                throw new \Exception($response);
            } else {
                throw new \Exception('No response from API : ' . curl_error($ch));
            }
        }

        return json_decode($response);
    }
}
